package com.qcloud.tencent_tpns_oppo_push_plugin;
import android.content.Context;
import androidx.annotation.NonNull;


import com.heytap.msp.push.HeytapPushManager;
import com.heytap.msp.push.callback.ICallBackResultService;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** FlutterOppoPushPlugin */
public class TencentTpnsOppoPushPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Context context;


  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    this.channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "flutter_oppo_push_plugin");
    channel.setMethodCallHandler(this);
    this.context = flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {

    if (call.method.equals("init")) {
      this.init(call,result);
    }else if(call.method.equals("isSupportPush")){
      this.isSupportPush(result);
    }else if(call.method.equals("register")){
      this.register(call,result);
    }else if(call.method.equals("getRegister")){
      this.getRegister(result);
    }else if(call.method.equals("setAppKeySecret")){
      this.setAppKeySecret(call,result);
    }else if(call.method.equals("unRegister")){
      this.unRegister(result);
    }
    else if(call.method.equals("pausePush")){
      this.pausePush(result);
    }
    else if(call.method.equals("resumePush")){
      this.resumePush(result);
    }
    else if(call.method.equals("getPushStatus")){
      this.getPushStatus(result);
    }
    else if(call.method.equals("getSDKVersion")){
      this.getSDKVersion(result);
    }
    else if(call.method.equals("getPushVersionCode")){
      this.getPushVersionCode(result);
    }
    else if(call.method.equals("getPushVersionName")){
      this.getPushVersionName(result);
    }
    else if(call.method.equals("setPushTime")){
      this.setPushTime(call,result);
    }
    else if(call.method.equals("requestNotificationPermission")){
      this.requestNotificationPermission(result);
    }
    else if(call.method.equals("openNotificationSetting")){
      this.openNotificationSetting(result);
    }
    else if(call.method.equals("getNotificationStatus")){
      this.getNotificationStatus(result);
    }
    else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }


  private void init(@NonNull MethodCall call,@NonNull Result result){

    boolean needLog = this.getParamByKey(call,"needLog");
    HeytapPushManager.init(context, needLog);
    result.success(null);
  }


  private void isSupportPush(@NonNull Result result){
    result.success(HeytapPushManager.isSupportPush());
  }

  private void register(@NonNull MethodCall call,@NonNull Result result){
    String appKey = this.getParamByKey(call,"appKey");
    String appSecret = this.getParamByKey(call,"appSecret");
    HeytapPushManager.register(context, appKey, appSecret, new ICallBackResultService() {
      @Override
      public void onRegister(int i, String s) {
        channel.invokeMethod("onRegister",makeCallBackData(i,s));
      }

      @Override
      public void onUnRegister(int i) {
        channel.invokeMethod("onUnRegister",makeCallBackData(i,null));
      }

      @Override
      public void onSetPushTime(int i, String s) {
        channel.invokeMethod("onSetPushTime",makeCallBackData(i,s));
      }

      @Override
      public void onGetPushStatus(int i, int i1) {
        channel.invokeMethod("onGetPushStatus",makeCallBackData(i,i1));
      }

      @Override
      public void onGetNotificationStatus(int i, int i1) {
        channel.invokeMethod("onGetNotificationStatus",makeCallBackData(i,i1));
      }
    });
    result.success(null);
  }
  private void setAppKeySecret(@NonNull MethodCall call,@NonNull Result result){
    String appKey = this.getParamByKey(call,"appKey");
    String appSecret = this.getParamByKey(call,"appSecret");
    HeytapPushManager.setAppKeySecret(appKey,appSecret);
    result.success(null);
  }


  private void unRegister(@NonNull Result result){
    HeytapPushManager.unRegister();
    result.success(null);
  }
  private void getRegister(@NonNull Result result){
    HeytapPushManager.getRegister();
    result.success(null);
  }
  private void pausePush(@NonNull Result result){
    HeytapPushManager.pausePush();
    result.success(null);
  }
  private void resumePush(@NonNull Result result){
    HeytapPushManager.resumePush();
    result.success(null);
  }
  private void getPushStatus(@NonNull Result result){
    HeytapPushManager.getPushStatus();
    result.success(null);
  }
  private void getSDKVersion(@NonNull Result result){
    result.success(HeytapPushManager.getSDKVersion());
  }
  private void getPushVersionCode(@NonNull Result result){
    result.success(HeytapPushManager.getPushVersionCode());
  }
  private void getPushVersionName(@NonNull Result result){
    result.success(HeytapPushManager.getPushVersionName());
  }
  private void setPushTime(@NonNull MethodCall call,@NonNull Result result){
    LinkedList<Integer> list = getParamByKey(call,"weekDays");
    int startHour = getParamByKey(call,"startHour");
    int endHour = getParamByKey(call,"endHour");
    int startMin  = getParamByKey(call,"startMin");
    int endMin  = getParamByKey(call,"endMin");
    HeytapPushManager.setPushTime(list,startHour,endHour,startMin,endMin);
    result.success(null);
  }
  private void requestNotificationPermission(@NonNull Result result){
    HeytapPushManager.requestNotificationPermission();
    result.success(null);
  }
  private void openNotificationSetting(@NonNull Result result){
    HeytapPushManager.openNotificationSettings();
    result.success(null);
  }
  private void getNotificationStatus(@NonNull Result result){
    HeytapPushManager.getNotificationStatus();
    result.success(null);
  }

  private <T> T getParamByKey(@NonNull MethodCall methodCall, String key) {
    T par = methodCall.argument(key);
    return par;
  }
  private Map<String,Object> makeCallBackData(int code,Object restData){
    HashMap<String,Object> data = new HashMap<String,Object>();
    data.put("responseCode",code);
    data.put("data",restData);
    return data;
  }
}
