import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class TencentTpnsOppoPushPlugin {
  ///nodoc
  static const MethodChannel _channel =
      const MethodChannel('flutter_oppo_push_plugin');

  ///nodoc
  static Function iCallBackResultService;

  ///设置注册回调
  ///
  static void setCallback({
    @required Function callback,
  }) {
    iCallBackResultService = callback;
    _channel.setMethodCallHandler((call) {
      iCallBackResultService(Map.from({
        "name": call.method,
        "responseCode": call.arguments['responseCode'],
        "data": call.arguments['data'],
      }));
      return;
    });
  }

  ///初始化OPPO PUSH服务，创建默认通道
  ///
  ///needLog是否需要设置log
  ///
  static void init({
    @required bool needLog,
  }) {
    _channel.invokeMethod("init", {
      "needLog": needLog,
    });
  }

  ///判断是否手机平台是否支持PUSH
  ///
  ///@return true 表示手机平台支持PUSH, false表示不支持
  ///
  static Future<bool> isSupportPush() async {
    return await _channel.invokeMethod("isSupportPush");
  }

  ///注册OPPO PUSH推送服务
  ///
  ///appKey 在开发者网站上注册时生成的，与AppKey相对应
  ///
  ///appSecret 与AppSecret相对应
  ///
  static void register({
    @required String appKey,
    @required String appSecret,
  }) {
    _channel.invokeMethod("register", {
      "appKey": appKey,
      "appSecret": appSecret,
    });
  }

  ///获取注册OPPO PUSH推送服务的注册ID
  ///
  static void getRegister() {
    _channel.invokeMethod("getRegister");
  }

  ///设置appKey等参数,可以覆盖register中的appkey设置
  ///
  ///appKey 在开发者网站上注册时生成的key
  ///
  ///appSecret 与AppSecret相对应
  ///
  static void setAppKeySecret({
    @required String appKey,
    @required String appSecret,
  }) {
    _channel.invokeMethod("setAppKeySecret", {
      "appKey": appKey,
      "appSecret": appSecret,
    });
  }

  ///解注册OPPO PUSH推送服务
  ///
  static void unRegister() {
    _channel.invokeMethod("unRegister");
  }

  ///暂停接收OPPO PUSH服务推送的消息
  ///
  static void pausePush() {
    _channel.invokeMethod("pausePush");
  }

  ///恢复接收OPPO PUSH服务推送的消息，这时服务器会把暂停时期的推送消息重新推送过来
  ///
  static void resumePush() {
    _channel.invokeMethod("pausePush");
  }

  ///获取OPPO PUSH推送服务状态
  ///
  static void getPushStatus() {
    _channel.invokeMethod("getPushStatus");
  }

  ///获取OPPO PUSH推送服务SDK版本（例如”2.1.0”）
  ///
  static Future<String> getSDKVersion() async {
    return await _channel.invokeMethod("getSDKVersion");
  }

  ///获取OPPO PUSH推送服务MCS版本(例如“2.4.0”)
  ///
  static Future<int> getPushVersionCode() async {
    return await _channel.invokeMethod("getPushVersionCode");
  }

  ///获取OPPO PUSH推送服务MCS名称(例如“2400”)
  ///
  static Future<String> getPushVersionName() async {
    return await _channel.invokeMethod("getPushVersionName");
  }

  ///设置允许推送时间 API
  ///
  ///weekDays 周日为0,周一为1,以此类推
  ///
  ///startHour 开始时间,24小时制
  ///
  ///endHour 结束时间,24小时制
  ///
  ///startMin 开始时间 分
  ///
  ///endMin 结束时间 分
  ///
  static void setPushTime({
    @required List<int> weekDays,
    @required int startHour,
    @required int endHour,
    @required int startMin,
    @required int endMin,
  }) {
    _channel.invokeMethod("setPushTime", {
      "weekDays": weekDays,
      "startHour": startHour,
      "endHour": endHour,
      "startMin": startMin,
      "endMin": endMin,
    });
  }

  /// 弹出通知栏权限弹窗（仅一次）
  ///
  static void requestNotificationPermission() {
    _channel.invokeMethod("requestNotificationPermission");
  }

  ///打开通知栏设置界面
  ///
  static void openNotificationSetting() {
    _channel.invokeMethod("openNotificationSetting");
  }

  ///获取通知栏状态，从callback回调结果
  ///
  static void getNotificationStatus() {
    _channel.invokeMethod("getNotificationStatus");
  }
}
